
import os

import tornado.web

import pyaas

from . import Base


class Index(Base):
    def initialize(self, template='index'):
        self.template = template + '.html'

    def get(self, template=None):
        template = template + '.html' if template else self.template
        self.render(template)


class Protected(Base):
    def initialize(self, template='index'):
        self.template = template + '.html'

    @tornado.web.authenticated
    def get(self, template=None):
        template = template + '.html' if template else self.template
        self.render(template)


class Source(Base):
    def get(self, src=''):
        srcdir = os.path.join(pyaas.prefix, 'src')
        src = os.path.join(srcdir, src)
        src = os.path.abspath(src)

        if not src.startswith(srcdir):
            raise tornado.web.HTTPError(404, 'Path escape attempt detected')

        try:
            data = open(src, 'rb').read()
        except:
            raise tornado.web.HTTPError(404, 'File not found')

        self.set_header('Content-Type', 'text/plain')
        self.write(data)




import os

import pyaas
import logging

try:
    import sqlite3
except ImportError:
    raise pyaas.error('Missing sqlite3 module')


class Sqlite:
    def __init__(self, path=None, schema=None):
        if path is None:
            path = ':memory:'
        else:
            path = pyaas.paths('var', path)
            # TODO: make any missing directories

        self.path = path
        self.schema = schema


    def Initialize(self):
        # Check if the db exists before we make a connection.  The connection
        # will create the file but it will not yet have a schema.
        exists = os.path.isfile(self.path)

        try:
            self.conn = sqlite3.connect(self.path)
            self.conn.text_factory = str
            self.conn.row_factory  = sqlite3.Row
            self.cursor = self.conn.cursor()
        except sqlite3.OperationalError:
            raise pyaas.error('Unable to open database: %s', self.path)

        if self.schema and not exists:
            schema = self.schema

            # Attempt to find the schema.  This is kind of dumb but it
            # prevents some old code from breaking.  At some point we
            # might could remove this first condition
            if not os.path.isfile(schema):
                schema = os.path.join(pyaas.prefix, self.schema)
            if not os.path.isfile(schema):
                schema = pyaas.paths('etc', self.schema)

            if os.path.isfile(schema):
                logging.debug('Attempt to load schema: %s', schema)
                schema = open(schema, 'rb').read()
                self.cursor.executescript(schema)
                self.conn.commit()
            else:
                logging.debug('Schema not found: %s', schema)

    def Cursor(self):
        return self.conn.cursor()

    def Close(self):
        self.conn.commit()
        self.conn.close()

    def Sync(self):
        self.conn.commit()

    def Find(self, table, params=None, sort=None):
        statement = 'SELECT * FROM ' + table
        if params:
            statement += ' WHERE ' + params
        if sort:
            statement += ' ' + sort

        self.cursor.execute(statement)
        return self.cursor.fetchall()

    def FindOne(self, table, _id, id_column='id'):
        statement = 'SELECT * FROM {0} WHERE {1} = ?'.format(table, id_column)
        self.cursor.execute(statement, [_id])
        return self.cursor.fetchone()

    def Count(self, table):
        statement = 'SELECT count(*) FROM {0}'.format(table)
        self.cursor.execute(statement)
        return self.cursor.fetchone()[0]

    def Insert(self, table, values, id_column='id'):
        columns = ','.join(values.keys())
        placeholder = ','.join('?' * len(values))
        statement = 'INSERT INTO {0} ({1}) VALUES ({2})'.format(table, columns, placeholder)

        try:
            self.cursor.execute(statement, values.values())
        except sqlite3.ProgrammingError as e:
            raise pyaas.error('Problem executing statement: %s', e)
        except sqlite3.IntegrityError as e:
            raise pyaas.error('Integrity error: %s', e)

        if id_column not in values:
            values[id_column] = self.cursor.lastrowid

        #self.conn.commit()

    def Update(self, table, values, id_column='id'):
        _id = values[id_column]
        columns = ','.join(s + '=?' for s in values.keys())
        statement = 'UPDATE {0} SET {1} WHERE id=?'.format(table, columns, _id)
        try:
            self.cursor.execute(statement, values.values() + [_id])
        except sqlite3.ProgrammingError:
            raise pyaas.error('Problem executing statement')

        #self.conn.commit()

    def Remove(self, table, _id, id_column='id'):
        statement = 'DELETE FROM {0} WHERE {1} = ?'.format(table, id_column)
        self.cursor.execute(statement, [_id])
        #self.conn.commit()




import sys

out = sys.stdout

class Colors:
    def black  (self, fmt='', *args): self('\x1b[1;30m'+fmt+'\x1b[0m', *args)
    def red    (self, fmt='', *args): self('\x1b[1;31m'+fmt+'\x1b[0m', *args)
    def green  (self, fmt='', *args): self('\x1b[1;32m'+fmt+'\x1b[0m', *args)
    def yellow (self, fmt='', *args): self('\x1b[1;33m'+fmt+'\x1b[0m', *args)
    def blue   (self, fmt='', *args): self('\x1b[1;34m'+fmt+'\x1b[0m', *args)
    def purple (self, fmt='', *args): self('\x1b[1;35m'+fmt+'\x1b[0m', *args)
    def cyan   (self, fmt='', *args): self('\x1b[1;36m'+fmt+'\x1b[0m', *args)
    def white  (self, fmt='', *args): self('\x1b[1;37m'+fmt+'\x1b[0m', *args)

class PrintF(Colors):
    def __call__(self, fmt='', *args):
        out.write(fmt % args)
        out.flush()

class WriteLine(Colors):
    def __call__(self, fmt='', *args):
        out.write(fmt % args)
        out.write('\n')

def hexdump(blob, width=16, offset=0):
    fmt = '%%.%dx: ' % len('%.x' % (len(blob) - 1))
    while blob:
        line = blob[:width]
        blob = blob[width:]

        printf.white(fmt, offset)
        printf.cyan(' '.join('%.2x' % ord(c) for c in line))
        printf(' ' * ((width-len(line))*3+1))

        for c in line:
            if ord(c) < 32 or ord(c) > 126:
                printf.black('.')
            else:
                printf.white('%c', c)

        writeln()
        offset += width

__builtins__['printf']  = PrintF()
__builtins__['writeln'] = WriteLine()
__builtins__['hexdump'] = hexdump
